@@ -7,6 +7,7 @@
  * - Jeff Davis <ruby-pg@j-davis.com>
  * - Guy Decoux (ts) <decoux@moulon.inra.fr>
  * - Michael Granger <ged@FaerieMUD.org>
+ * - Lars Kanis <lars@greiz-reinsdorf.de>
  * - Dave Lee
  * - Eiji Matsumoto <usagi@ruby.club.or.jp>
  * - Yukihiro Matsumoto <matz@ruby-lang.org>
@@ -15,10 +16,10 @@
  * See Contributors.rdoc for the many additional fine people that have contributed
  * to this library over the years.
  *
- * Copyright (c) 1997-2012 by the authors.
+ * Copyright (c) 1997-2016 by the authors.
  *
  * You may redistribute this software under the same terms as Ruby itself; see
- * http://www.ruby-lang.org/en/LICENSE.txt or the LICENSE file in the source
+ * https://www.ruby-lang.org/en/about/license.txt or the BSDL file in the source
  * for details.
  *
  * Portions of the code are from the PostgreSQL project, and are distributed
@@ -47,6 +48,7 @@
 
 #include "pg.h"
 
+int pg_skip_deprecation_warning;
 VALUE rb_mPG;
 VALUE rb_mPGconstants;
 
@@ -68,7 +70,6 @@ VALUE rb_mPGconstants;
  * M17n functions
  */
 
-#ifdef M17N_SUPPORTED
 /**
  * The mapping from canonical encoding names in PostgreSQL to ones in Ruby.
  */
@@ -123,24 +124,6 @@ const char * const (pg_enc_pg2ruby_mapping[][2]) = {
  * A cache of mapping from PostgreSQL's encoding indices to Ruby's rb_encoding*s.
  */
 static struct st_table *enc_pg2ruby;
-static ID s_id_index;
-
-
-/*
- * Get the index of encoding +val+.
- * :FIXME: Look into replacing this with rb_enc_get_index() since 1.9.1 isn't really
- * used anymore.
- */
-int
-pg_enc_get_index(VALUE val)
-{
-	int i = ENCODING_GET_INLINED(val);
-	if (i == ENCODING_INLINE_MAX) {
-		VALUE iv = rb_ivar_get(val, s_id_index);
-		i = NUM2INT(iv);
-	}
-	return i;
-}
 
 
 /*
@@ -160,9 +143,6 @@ pg_find_or_create_johab(void)
 	}
 
 	enc_index = rb_define_dummy_encoding(aliases[0]);
-	for (i = 1; i < sizeof(aliases)/sizeof(aliases[0]); ++i) {
-		ENC_ALIAS(aliases[i], aliases[0]);
-	}
 	return rb_enc_from_index(enc_index);
 }
 
@@ -246,14 +226,55 @@ pg_get_rb_encoding_as_pg_encoding( rb_encoding *enc )
 	return encname;
 }
 
-#endif /* M17N_SUPPORTED */
+
+/*
+ * Ensures that the given string has enough capacity to take expand_len
+ * more data bytes. The new data part of the String is not initialized.
+ *
+ * current_out must be a pointer within the data part of the String object.
+ * This pointer is returned and possibly adjusted, because the location of the data
+ * part of the String can change through this function.
+ *
+ * PG_RB_STR_ENSURE_CAPA can be used to do fast inline checks of the remaining capacity.
+ * end_capa it is then set to the first byte after the currently reserved memory,
+ * if not NULL.
+ *
+ * Before the String can be used with other string functions or returned to Ruby space,
+ * the string length has to be set with rb_str_set_len().
+ *
+ * Usage example:
+ *
+ *   VALUE string;
+ *   char *current_out, *end_capa;
+ *   PG_RB_STR_NEW( string, current_out, end_capa );
+ *   while( data_is_going_to_be_processed ){
+ *     PG_RB_STR_ENSURE_CAPA( string, 2, current_out, end_capa );
+ *     *current_out++ = databyte1;
+ *     *current_out++ = databyte2;
+ *   }
+ *   rb_str_set_len( string, current_out - RSTRING_PTR(string) );
+ *
+ */
+char *
+pg_rb_str_ensure_capa( VALUE str, long expand_len, char *curr_ptr, char **end_ptr )
+{
+	long curr_len = curr_ptr - RSTRING_PTR(str);
+	long curr_capa = rb_str_capacity( str );
+	if( curr_capa < curr_len + expand_len ){
+		rb_str_set_len( str, curr_len );
+		rb_str_modify_expand( str, (curr_len + expand_len) * 2 - curr_capa );
+		curr_ptr = RSTRING_PTR(str) + curr_len;
+	}
+	if( end_ptr )
+		*end_ptr = RSTRING_PTR(str) + rb_str_capacity( str );
+	return curr_ptr;
+}
 
 
 /**************************************************************************
  * Module Methods
  **************************************************************************/
 
-#ifdef HAVE_PQLIBVERSION
 /*
  * call-seq:
  *   PG.library_version -> Integer
@@ -271,7 +292,6 @@ pg_s_library_version(VALUE self)
 	UNUSED( self );
 	return INT2NUM(PQlibVersion());
 }
-#endif
 
 
 /*
@@ -289,6 +309,67 @@ pg_s_threadsafe_p(VALUE self)
 	return PQisthreadsafe() ? Qtrue : Qfalse;
 }
 
+static int
+pg_to_bool_int(VALUE value)
+{
+	switch( TYPE(value) ){
+		case T_FALSE:
+			return 0;
+		case T_TRUE:
+			return 1;
+		default:
+			return NUM2INT(value);
+	}
+}
+
+/*
+ * call-seq:
+ *    PG.init_openssl(do_ssl, do_crypto)  -> nil
+ *
+ * Allows applications to select which security libraries to initialize.
+ *
+ * If your application initializes libssl and/or libcrypto libraries and libpq is
+ * built with SSL support, you should call PG.init_openssl() to tell libpq that the
+ * libssl and/or libcrypto libraries have been initialized by your application,
+ * so that libpq will not also initialize those libraries. See
+ * http://h71000.www7.hp.com/doc/83final/BA554_90007/ch04.html for details on the SSL API.
+ *
+ * When do_ssl is +true+, libpq will initialize the OpenSSL library before first
+ * opening a database connection. When do_crypto is +true+, the libcrypto library
+ * will be initialized. By default (if PG.init_openssl() is not called), both libraries
+ * are initialized. When SSL support is not compiled in, this function is present but does nothing.
+ *
+ * If your application uses and initializes either OpenSSL or its underlying libcrypto library,
+ * you must call this function with +false+ for the appropriate parameter(s) before first opening
+ * a database connection. Also be sure that you have done that initialization before opening a
+ * database connection.
+ *
+ */
+static VALUE
+pg_s_init_openssl(VALUE self, VALUE do_ssl, VALUE do_crypto)
+{
+	UNUSED( self );
+	PQinitOpenSSL(pg_to_bool_int(do_ssl), pg_to_bool_int(do_crypto));
+	return Qnil;
+}
+
+
+/*
+ * call-seq:
+ *    PG.init_ssl(do_ssl)  -> nil
+ *
+ * Allows applications to select which security libraries to initialize.
+ *
+ * This function is equivalent to <tt>PG.init_openssl(do_ssl, do_ssl)</tt> . It is sufficient for
+ * applications that initialize both or neither of OpenSSL and libcrypto.
+ */
+static VALUE
+pg_s_init_ssl(VALUE self, VALUE do_ssl)
+{
+	UNUSED( self );
+	PQinitSSL(pg_to_bool_int(do_ssl));
+	return Qnil;
+}
 
 
 /**************************************************************************
@@ -298,19 +379,28 @@ pg_s_threadsafe_p(VALUE self)
 void
 Init_pg_ext()
 {
+	if( RTEST(rb_eval_string("ENV['PG_SKIP_DEPRECATION_WARNING']")) ){
+		/* Set all bits to disable all deprecation warnings. */
+		pg_skip_deprecation_warning = 0xFFFF;
+	} else {
+		pg_skip_deprecation_warning = 0;
+	}
+
 	rb_mPG = rb_define_module( "PG" );
 	rb_mPGconstants = rb_define_module_under( rb_mPG, "Constants" );
 
 	/*************************
 	 *  PG module methods
 	 *************************/
-#ifdef HAVE_PQLIBVERSION
 	rb_define_singleton_method( rb_mPG, "library_version", pg_s_library_version, 0 );
-#endif
 	rb_define_singleton_method( rb_mPG, "isthreadsafe", pg_s_threadsafe_p, 0 );
 	SINGLETON_ALIAS( rb_mPG, "is_threadsafe?", "isthreadsafe" );
 	SINGLETON_ALIAS( rb_mPG, "threadsafe?", "isthreadsafe" );
 
+  rb_define_singleton_method( rb_mPG, "init_openssl", pg_s_init_openssl, 2 );
+  rb_define_singleton_method( rb_mPG, "init_ssl", pg_s_init_ssl, 1 );
+
+
 	/******     PG::Connection CLASS CONSTANTS: Connection Status     ******/
 
 	/* Connection succeeded */
@@ -368,7 +458,6 @@ Init_pg_ext()
 	/* Verbose error verbosity level (#set_error_verbosity) */
 	rb_define_const(rb_mPGconstants, "PQERRORS_VERBOSE", INT2FIX(PQERRORS_VERBOSE));
 
-#ifdef HAVE_PQPING
 	/******     PG::Connection CLASS CONSTANTS: Check Server Status ******/
 
 	/* Server is accepting connections. */
@@ -379,7 +468,6 @@ Init_pg_ext()
 	rb_define_const(rb_mPGconstants, "PQPING_NO_RESPONSE", INT2FIX(PQPING_NO_RESPONSE));
 	/* Connection not attempted (bad params). */
 	rb_define_const(rb_mPGconstants, "PQPING_NO_ATTEMPT", INT2FIX(PQPING_NO_ATTEMPT));
-#endif
 
 	/******     PG::Connection CLASS CONSTANTS: Large Objects     ******/
 
@@ -414,13 +502,9 @@ Init_pg_ext()
 	/* #result_status constant: A fatal error occurred. */
 	rb_define_const(rb_mPGconstants, "PGRES_FATAL_ERROR", INT2FIX(PGRES_FATAL_ERROR));
 	/* #result_status constant: Copy In/Out data transfer in progress. */
-#ifdef HAVE_CONST_PGRES_COPY_BOTH
 	rb_define_const(rb_mPGconstants, "PGRES_COPY_BOTH", INT2FIX(PGRES_COPY_BOTH));
-#endif
 	/* #result_status constant: Single tuple from larger resultset. */
-#ifdef HAVE_CONST_PGRES_SINGLE_TUPLE
 	rb_define_const(rb_mPGconstants, "PGRES_SINGLE_TUPLE", INT2FIX(PGRES_SINGLE_TUPLE));
-#endif
 
 	/******     Result CONSTANTS: result error field codes      ******/
 
@@ -532,14 +616,25 @@ Init_pg_ext()
 	/* Add the constants to the toplevel namespace */
 	rb_include_module( rb_mPG, rb_mPGconstants );
 
-#ifdef M17N_SUPPORTED
 	enc_pg2ruby = st_init_numtable();
-	s_id_index = rb_intern("@encoding");
-#endif
 
 	/* Initialize the main extension classes */
 	init_pg_connection();
 	init_pg_result();
 	init_pg_errors();
+	init_pg_type_map();
+	init_pg_type_map_all_strings();
+	init_pg_type_map_by_class();
+	init_pg_type_map_by_column();
+	init_pg_type_map_by_mri_type();
+	init_pg_type_map_by_oid();
+	init_pg_type_map_in_ruby();
+	init_pg_coder();
+	init_pg_text_encoder();
+	init_pg_text_decoder();
+	init_pg_binary_encoder();
+	init_pg_binary_decoder();
+	init_pg_copycoder();
+	init_pg_tuple();
 }
 
