@@ -1,4 +1,4 @@
-#!/usr/bin/env ruby
+# -*- ruby -*-
 
 begin
 	require 'pg_ext'
@@ -7,7 +7,23 @@ rescue LoadError
 	if RUBY_PLATFORM =~/(mswin|mingw)/i
 		major_minor = RUBY_VERSION[ /^(\d+\.\d+)/ ] or
 			raise "Oops, can't extract the major/minor version from #{RUBY_VERSION.dump}"
-		require "#{major_minor}/pg_ext"
+
+		add_dll_path = proc do |path, &block|
+			begin
+				require 'ruby_installer/runtime'
+				RubyInstaller::Runtime.add_dll_directory(path, &block)
+			rescue LoadError
+				old_path = ENV['PATH']
+				ENV['PATH'] = "#{path};#{old_path}"
+				block.call
+				ENV['PATH'] = old_path
+			end
+		end
+
+		# Temporary add this directory for DLL search, so that libpq.dll can be found.
+		add_dll_path.call(__dir__) do
+			require "#{major_minor}/pg_ext"
+		end
 	else
 		raise
 	end
@@ -19,7 +35,7 @@ end
 module PG
 
 	# Library version
-	VERSION = '0.17.0'
+	VERSION = '1.1.3'
 
 	# VCS revision
 	REVISION = %q$Revision$
@@ -43,12 +59,16 @@ module PG
 
 	require 'pg/exceptions'
 	require 'pg/constants'
+	require 'pg/coder'
+	require 'pg/binary_decoder'
+	require 'pg/text_encoder'
+	require 'pg/text_decoder'
+	require 'pg/basic_type_mapping'
+	require 'pg/type_map_by_column'
 	require 'pg/connection'
 	require 'pg/result'
+	require 'pg/tuple'
 
 end # module PG
 
 
-# Backward-compatible aliase
-PGError = PG::Error
-
